/*
 * FreqCounter_Test.cpp
 *
 * aktuelle Hardware AtMega328P mit 8MHz (interner RC-Oszillator), UART0
 *
 * Hardware: ATMega328 oder ATMega1284 oder ATMega2561
 * CPU-Takt 16 oder 8MHz
 *
 * (abgeschaltet) Timer 0 erzeugt an Pin OC0B ein 5 kHz Rechecksignal PD5 (ATMega328) bzw. PB4 (ATMega1284) bzw. PG5 (ATMega2561) 
 * Timer 2 sorgt fr die Torzeit (gatetime)
 * Timer 1 misst Frequenz via T1 -> PD5 bei AtMega328P oder PB1 bei AtMega1284
 *
 * luft bei 8/16 MHz Takt! Aktuell: 8 MHz interner RC Oszillator ist eingestellt
 * bei 125 kHz Timer1-Takt Messbereich 100Hz...3MHz
 *
 * Spielerei mit Watchdog
 *
 * Created: 30.03.2020 13:01:50
 * Author : Micha
 */ 

/* nderung bei den Parametern des Linkers
  Zur Benutzung von sprintf() und float / double-Variablen - nderungen beim Linker:
- Men Projekt, Properties zu diesem Projekt aufrufen
- unter Toolchain / Linker / General die Option  "Use vprintf-library" anklicken
- unter Toolchain / Linker / Miscellaneous in das Feld eintragen: -lprintf_flt
- nderungen speichern (STRG-S)
- nun kann man sprintf() auch benutzen mit float und double
- Quelle: https://startingelectronics.org/articles/atmel-AVR-8-bit/print-float-atmel-studio-7/
*/

#ifndef F_CPU
//#define F_CPU 16000000UL								// Takt 16 MHz externer Quarz
#define F_CPU 8000000UL								// Takt 8 MHz interner RC-Oszillator
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "uart0.h"								// fr ATMega328P
//#include "uart1.h"							// alternativ fr ATMega1284P etc.
#include <avr/wdt.h>							// Watchdog
#include "FreqCounter.h"

#define messtiefe 10
long frequenzen[messtiefe];
long frq;										// eine gemessene Frequenz

char s[40] = "";

//void generate_5000Hz()							// timer0
//{
	////generate a 5 kHz square wave on pin OC0B
	////DDRD |= _BV(PD5);							//PD5 = OC0B as Output (AtMega328)
	//// - oder -
	////DDRB |= _BV(PB4);							//PB4 = OC0B as Output (Atmega1284)
	////DDRG |= _BV(PG5);							//PG5 = OC0B as Output (AtMega2561)
	//
	//TCCR0B = 0;									//stop timer
	//TIMSK0 = 0;									//disable overflow interrupt
	//OCR0A = 50 - 1;								//period = 50 * 64 cycles
	//OCR0B = 50 / 2 - 1;							//50% duty cycle
	//TCCR0A = _BV(COM0B1)						//non-inverting PWM on OC0B
	//| _BV(WGM00)								//fast PWM, TOP = OCR0A
	//| _BV(WGM01);								//...ditto
	//TCCR0B = _BV(WGM02)							//...ditto
	//| _BV(CS00)									//clock at F_CPU / 64
	//| _BV(CS01);								//...ditto
//}

// Misst mehrfach die Frequenz und ermittelt, ob gesuchte Frequenz darin enthalten ist.
// gibt die Anzahl der Messungen zurck, die im gesuchten Bereich liegen.
uint8_t Frequenz_messen(long wanted_frequency)
{
	int i;
	long f_untergrenze = wanted_frequency - (wanted_frequency / 20);	// - 5%
	long f_obergrenze = wanted_frequency + (wanted_frequency / 20);		// + 5%
	int8_t result = 0;
	
	for (i=0;i<messtiefe;i++)
	{
		FreqCounter::start(100);						// Dauer der Messung in ms
		while (FreqCounter::f_ready == 0);				// warten auf Ende der Messung
		frequenzen[i] = FreqCounter::f_freq;
	}
	for (i=0;i < messtiefe; i++)
	{
		if ( (frequenzen[i] > f_untergrenze) && (frequenzen[i] < f_obergrenze))
		{
			result++;
		}
		sprintf(s, " Nr. %d Freq.: %ld \r\n", i, frequenzen[i]);				// %ld ist long integer
		UartSendString(s);
	}
	return result;
}


void setup()
{
	DDRD  &= ~(1 << PD5);						// Input -> PortD5/T1 (Atmega328P) 
	wdt_enable(WDTO_2S);						// Spielerei mit Watchdog, Timout 2 Sekunden
	//wdt_disable();
	USART0_Init();
	sei();
	sprintf(s, " - Frequenzzhler -\r\n");
	UartSendString(s);
}    

    
int main(void)
{
	setup();
	
	while (1)
	{
		// aktivieren, wenn man z.B. eine Frequenz von 4000Hz sucht...
		//sprintf(s, " - Freq erkannt: %d \r\n", Frequenz_messen(4000) );
		//UartSendString(s);

		// Frequenz 100ms messen		
		FreqCounter::start(100);						// Dauer der Messung in ms (Torzeit)
		while (FreqCounter::f_ready == 0);				// warten auf Ende der Messung
		frq = FreqCounter::f_freq;
		sprintf(s, " - Freq: %ld \r\n", frq );
		UartSendString(s);
		_delay_ms(1000);								// Zeit verschwenden, damit Terminalprogramm nicht volluft
		wdt_reset();									// WDT zurcksetzen, verhindert System Reset
	}
 }
