/*
 * uart0.h
 *
 * Created: 03.04.2020 23:01:47
 *  Author: Micha
 */ 


#ifndef UART0_H_
#define UART0_H_

//----------------------------------------------------------------------------------------------------------
// USART:
//
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)
//char s[40];
volatile char rx_value;
char dummy_char;


// USART0
void USART0_Init(void)
{
   // DDRD  |= (1 << 1);					// set Bit PD1 -> TxD = Ausgang (Atmega1284P)
   // Set baud rate
   UBRR0L = BAUD_PRESCALE;// Load lower 8-bits into the low byte of the UBRR register
   UBRR0H = (BAUD_PRESCALE >> 8); 
	/* Load upper 8-bits into the high byte of the UBRR register
    Default frame format is 8 data bits, no parity, 1 stop bit
  to change use UCSRC, see AVR datasheet */ 

  // Enable receiver and transmitter and receive complete interrupt 
  UCSR0B = ((1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0));
}

ISR(USART_RX_vect)
{
	rx_value = UDR0;				//read UART register into value
}

void UartSendString(char data[])   //sendet Char-Array
{
	uint8_t i = 0;
	while(!(UCSR0A & (1<<UDRE0)))
	{
		;//Do nothing
	}
	
	while (data[i] != '\0')
	{
		UDR0 = data[i];
		while ( !( UCSR0A & (1 << UDRE0)) )
		;
		i++;
	}
}

 // ein Zeichen senden
 void UARTWriteChar(char data)
{
	//Wait until the transmitter is ready

	while(!(UCSR0A & (1<<UDRE0)))
	{
		;//nichts machen, nur warten
	}

	//Now write the data to USART buffer

	UDR0=data;
}

// Wait until a byte has been received and return received data
uint8_t UART_ReceiveByte()
{
	while((UCSR0A &(1<<RXC0)) == 0);
	return UDR0;
}

// ein Zeichen lesen
char UARTReadChar()
{
	//Wait until a data is available

	while(!(UCSR0A & (1<<RXC0)))
	{
		//Do nothing
	}

	//Now USART has got data from host
	//and is available in buffer

	return UDR0;
}
//----------------------------------------------------------------------------------------------------------

// Anwendung:
// char s [40];
// char adr_buffer [sizeof(uint32_t)*8+1];

// ultoa (adr,adr_buffer,10);				// unsigned long to ASCII
// sprintf(s,"%s",adr_buffer);				// in Char Array

// USART0_sendString((char*)s);
// USART0_sendString((char*)" -> ");
// sprintf(s,"EEPROM write %ld data: %d \r\n",eeprom_address, data_to_write);
// USART0_sendString((char*)s);

// printf() Format Specifiers
// Erklrung zu sprintf() und Formate: https://efundies.com/avr-and-printf/




#endif /* UART0_H_ */