/*
 * uart1.h
 *
 * Created: 26.03.2020 10:04:01
 *  Author: Micha
 */ 


#ifndef UART1_H_
#define UART1_H_

//----------------------------------------------------------------------------------------------------------
// USART1:
//
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)
char dummy_char;
volatile char rx_value;


void USART1_Init(void)
{
	//DDRD  |= (1 << 3);				// set Bit PD3 -> TxD1 = Ausgang
	//DDRD  &= ~(1 << 2);				// reset Bit PD2 -> RxD1 = Eingang
	// Set baud rate
	UBRR1L = BAUD_PRESCALE;				// Load lower 8-bits into the low byte of the UBRR register
	UBRR1H = (BAUD_PRESCALE >> 8);		// Load upper 8-bits into the high byte of the UBRR register
    //Default frame format is 8 data bits, no parity, 1 stop bit
	//to change use UCSRC, see AVR datasheet */ 

  // Enable receiver and transmitter and receive complete interrupt 
  UCSR1B = ((1<<TXEN1) | (1<<RXEN1) | (1<<RXCIE1));
}


ISR(USART1_RX_vect)
{
	rx_value = UDR1;				//read UART register into value
}

void sendData(char data[])			//sendData 
{
	uint8_t i = 0;
	while(!(UCSR1A & (1<<UDRE1)))
	{
		;//Do nothing
	}
	
	while (data[i] != '\0')
	{
		UDR1 = data[i];
		while ( !( UCSR1A & (1 << UDRE1)) )
		;
		i++;
	}
}

 // ein Zeichen senden
 void UARTWriteChar(char data)
{
	//Wait until the transmitter is ready

	while(!(UCSR1A & (1<<UDRE1)))
	{
		;//nichts machen, nur warten
	}

	//Now write the data to USART buffer

	UDR1=data;
}


void USART1_sendString(char *str)
{
	for(size_t i = 0; i < strlen(str); i++)
	{
		UARTWriteChar(str[i]);
	}
}

// Wait until a byte has been received and return received data
uint8_t UART_ReceiveByte(){
	while((UCSR1A &(1<<RXC1)) == 0);
	return UDR1;
}

// ein Zeichen lesen
char UARTReadChar()
{
	//Wait until a data is available

	while(!(UCSR1A & (1<<RXC1)))
	{
		//Do nothing
	}

	//Now USART has got data from host
	//and is available in buffer

	return UDR1;
}
//----------------------------------------------------------------------------------------------------------


// Debugging: eine Variable anzeigen
void debug_int(uint16_t wert)
{
	uint8_t i= 0;
	char s[10] = "";
	// char * itoa(int val,char * s,int radix)
	itoa(wert,s,10);
	for(uint8_t i=0;i<9;i++)			// CR an die Ziffern anhngen
	{
		//if (s[i] == 0)
		//{
		//s[i]= 13;
		//s[i+1] = 10;
		//break;
		//}
	}
	do
	{
		UARTWriteChar(s[i]);	// wait until TX can send
	} while (s[++i] != 0);
}


// sendet String
void debugSendString(char *str)
{
	for(size_t i = 0; i < strlen(str); i++)
	{
		UARTWriteChar(str[i]);
	}
}

// wie debugSend: char array
void debug_s(char *s)
{
	volatile int i = 0;
	do
	{
		UARTWriteChar(s[i]);		// wait until TX can send
	} while (s[++i] != 0);
}


// Anwendung:
// char s [40];
// char adr_buffer [sizeof(uint32_t)*8+1];

// ultoa (adr,adr_buffer,10);				// unsigned long to ASCII
// sprintf(s,"%s",adr_buffer);				// in Char Array

// USART0_sendString((char*)s);
// USART0_sendString((char*)" -> ");
// sprintf(s,"EEPROM write %ld data: %d \r\n",eeprom_address, data_to_write);
// USART0_sendString((char*)s);

// printf() Format Specifiers
// Erklrung zu sprintf() und Formate: https://efundies.com/avr-and-printf/


#endif /* UART1_H_ */